@@ -9,7 +9,7 @@
 
 ;; GNU Emacs is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 2, or (at your option)
+;; the Free Software Foundation; either version 3, or (at your option)
 ;; any later version.
 
 ;; GNU Emacs is distributed in the hope that it will be useful,
@@ -170,7 +170,7 @@ Enter as a sexp.  Examples: \"\\C-z\", [(control ?z)]."
 	 (let ((old-value (if (boundp 'viper-toggle-key)
 			      viper-toggle-key
 			    [(control ?z)])))
-	   (mapcar
+	   (mapc
 	    (lambda (buf)
 	      (save-excursion
 		(set-buffer buf)
@@ -210,7 +210,7 @@ If running in a terminal, [(escape)] is not understood, so must use \"\\e\"."
 	 (let ((old-value (if (boundp 'viper-ESC-key)
 			      viper-ESC-key
 			    [(escape)])))
-	   (mapcar
+	   (mapc
 	    (lambda (buf)
 	      (save-excursion
 		(set-buffer buf)
